let config = require('./configReader.js')
if (!config.tasks.js) return

let path = require('path')
let pathToUrl = require('./pathToUrl')
let webpack = require('webpack')
let webpackManifest = require('./webpackManifest')

module.exports = function (env) {
    const jsSrc = path.resolve(config.root.src, config.tasks.js.src);
    const nodeModuleSrc = 'node_modules';
    const jsDest = path.resolve(config.root.dest, config.tasks.js.dest);
    const publicPath = pathToUrl(config.tasks.js.dest, '/');
    let extensions = config.tasks.js.extensions.map(function (extension) {
        return '.' + extension
    });

    let rev = config.tasks.production.rev && env === 'production'
    const filenamePattern = rev ? '[name]-[hash].js' : '[name].js'

    let plugins = config.tasks.js.plugins.map(function (plugin) {
        let pluginConfig = {};
        pluginConfig[plugin.reference] = plugin.name;
        return new webpack.ProvidePlugin(pluginConfig);
    });

    // `config.tasks.js.targetBuildVariables`
    // Usage:
    // 1. List the build-target-specific variables you like to use in your scripts, in tasks.js.targetBuildVariables
    // 2.A Define new npm scripts for all the target-environments you like to have, Environment-Variables to a specific value.
    // or: 
    // 2.B Just set the Environment-Variables (E.g. 'API_USERS') directli via CLI (plattform-specific command)
    // 3. Use the variables in your javascript files. Note that 'API_USERS' will be available as __API_USERS__ in your Script.
    // 
    // 
    // Basic scenario would be that you want to have different build, which test different API's, for instance:
    // Test-API on: "test.example.com"
    // Develop-API on: "dev.example.com"
    //
    // Example package.JSON: (Only with specific 'develop' script. You probably have to define the other Scripts like 
    // "production_test", "production_develop" etc. as well.)
    // `
    //     "tasks": {
    //       "js": {
    //         "targetBuildVariables": [
    //           "API_USERS",
    //           "API_PRODUCTS"
    //         ]
    //       }
    //     },
    //     "scripts": {
    //        "develop": "gulp --gulpfile node_modules/viu-launchpad/gulpfile.js/index.js",
    //        "develop_test": "cross-env API_USERS=test.example.com/getusers API_PRODUCTS=test.example.com/getproducts npm run develop",
    //        "develop_dev": "cross-env API_USERS=dev.example.com/getusers API_PRODUCTS=dev.example.com/getproducts npm run develop"
    //     }
    // `
    //
    // IMPORTANT: Note the `cross-env` call at the beginning of the npm-scripts `develop_test` and `develop_dev`. The `cross-env` package will make sure that setting the environment-variable (e.g. `API_USERS`) will work cross-platform.
    // 
    // With this configuration, you can use your custom variables in your javascript files like following:
    // 
    // `
    // console.log(__API_USERS__);
    // console.log(__API_PRODUCTS__);
    // `
    // 
    // `npm run develop_test` 
    // => 'test.example.com/getusers'
    // => 'test.example.com/getproducts'
    // 
    // `npm run develop_dev`
    // => 'dev.example.com/getusers'
    // => 'dev.example.com/getproducts'
    //
    // Note: If you have set the environment variable directly via CLI, you just would run 'npn run develop (or whatever)'.
    // 
    plugins = [
        ...plugins,
        ...config.tasks.js.targetBuildVariables.map(targetBuildVariable => (
                new webpack.DefinePlugin({
                    [`__${targetBuildVariable}__`]: JSON.stringify(process.env[targetBuildVariable])
                })
            ))
    ];

    config.tasks.js.contextReplacementPlugins.map(
        plugin => plugins.push(new webpack.ContextReplacementPlugin(new RegExp(plugin.context), new RegExp(plugin.replacement)))
    );

    let loaders = [
    ];

    if (config.tasks.js.loaders) {
        for (let i = 0; i < config.tasks.js.loaders.length; i++) {
            let loader = config.tasks.js.loaders[i];
            loader.test = new RegExp(loader.test, 'i');
            loaders.push(loader);
        }
    }

    let webpackConfig = {
        context: jsSrc,
        plugins: plugins,
        resolve: {
            modules: [
                jsSrc,
                nodeModuleSrc,
            ],
            extensions: extensions
        },
        module: {
            rules: loaders
        }
    };

    if (env === 'development') {
        webpackConfig.devtool = 'inline-source-map'

        // Create new entries object with webpack-hot-middleware added
        for (var key in config.tasks.js.entries) {
            var entry = config.tasks.js.entries[key]
            config.tasks.js.entries[key] = ['webpack-hot-middleware/client?&reload=true'].concat(entry)
        }

        webpackConfig.plugins.push(new webpack.HotModuleReplacementPlugin())
    }

    if (env !== 'test') {
        // Karma doesn't need entry points or output settings
        webpackConfig.entry = config.tasks.js.entries

        webpackConfig.output = {
            path: path.normalize(jsDest),
            filename: filenamePattern,
            publicPath: publicPath
        };

        if (config.tasks.js.extractSharedJs) {
            // Factor out common dependencies into a shared.js
            webpackConfig.plugins.push(
                new webpack.optimize.CommonsChunkPlugin({
                    name: 'shared',
                    filename: filenamePattern,
                })
            )
        }
    }

    if (env === 'production') {
        if (rev) {
            webpackConfig.plugins.push(new webpackManifest(publicPath, config.root.dest))
        }
        webpackConfig.plugins.push(
            // LoaderOptions Plugin to be deprecated in Webpack 3
            // However, as not all loaders are updated for Webpack 2,
            // We include this as a default option for the launchpad here
            // TODO, remove this option in the future.
            new webpack.LoaderOptionsPlugin({
                minimize: true
            }),
            new webpack.DefinePlugin({
                'process.env': {
                    'NODE_ENV': JSON.stringify('production')
                }
            }),
            new webpack.optimize.UglifyJsPlugin({
                mangle: config.tasks.js.mangle
            }),
            new webpack.NoEmitOnErrorsPlugin()
        )
    }

    if (env === 'debug') {
        webpackConfig.devtool = 'inline-source-map';
        if (rev) {
            webpackConfig.plugins.push(new webpackManifest(publicPath, config.root.dest))
        }
        webpackConfig.plugins.push(
            new webpack.DefinePlugin({
                'process.env': {
                    'NODE_ENV': JSON.stringify('production')
                }
            }),
            new webpack.NoEmitOnErrorsPlugin()
        )
    }

    // Set additional configurations in webpackConfig
    // Attention: it's possible to overwrite all previously set properties in webpackConfig with this setting
    for (let key in config.tasks.js.additionalWebpackSettings) {
        webpackConfig[key] = config.tasks.js.additionalWebpackSettings[key];
    }

    return webpackConfig
};
